@@ -1,4 +1,4 @@
-__version__ = "0.0.4"
+__version__ = "0.0.5"
 
 from .core import (
     broadcast_batch,
@@ -9,6 +9,7 @@ from .core import (
     to_34,
     look_at_np,
     to_np,
+    to_numpy,
     to_torch,
     to_8b,
     to_float,
@@ -22,8 +23,11 @@ from .core import (
 )
 
 from .geometry_basic import (
+    duplicate_faces,
+    remove_duplicate_faces,
     get_aspect_ratio,
     normalize_vertices,
+    clean_infinite_vertices,
     calc_edges,
     calc_face_normals,
     calc_vertex_normals,
@@ -31,13 +35,16 @@ from .geometry_basic import (
     get_center_of_attention,
     scale_poses,
     find_princple_componenets,
+    rotmat2qvec,
+    qvec2rotmat,
+    qslerp
 )
 
 from .geometry_advanced import (
     distribute_field,
     distribute_scalar_field,
     distribute_vector_field,
-    qslim
+    qem
 )
 
 from .gsoup_io import (
